@@ -2,13 +2,13 @@ package Treex::PML::Backend::CSTS;
 
 use vars qw($VERSION);
 BEGIN {
-  $VERSION='2.015'; # version template
+  $VERSION='2.02'; # version template
 }
 
 use Treex::PML;
 use Treex::PML::IO qw(set_encoding);
-use Treex::PML::Backend::CSTS::Csts2fs;
-use Treex::PML::Backend::CSTS::Fs2csts;
+require Treex::PML::Backend::CSTS::Csts2fs;
+require Treex::PML::Backend::CSTS::Fs2csts;
 use Fcntl qw(SEEK_SET);
 use File::ShareDir;
 
@@ -89,11 +89,11 @@ sub close_backend {
 }
 
 sub read {
-  Treex::PML::Backend::CSTS::Csts2fs::read(@_);
+  Csts2fs::read(@_);
 }
 
 sub write {
-  Treex::PML::Backend::CSTS::Fs2csts::write(@_);
+  Fs2csts::write(@_);
 }
 
 sub test_nsgmls {
